// SchoolApp.java -  Client program for Student MDB
import java.io.*;
import javax.jms.*;
import javax.naming.*;
import asg.Student.StudentBean;

public class SchoolApp implements MessageListener {

	private TopicConnection topicConnect = null;
	private Topic courseTopic = null;
	private TopicPublisher publisher = null;
	private TopicSession publishSession = null;
	private QueueConnection queueConnect = null;
	private Queue receiveQueue = null;
	private QueueSession queueSession = null;

	/******************************************************
	 *  Sets up topic connection and topic session objects 
	 *  for publisher-subscribe messaging
	 *  Sets up queue connection and queue session objects
	 *  for point-to-point reply messaging
	 *  Creates publisher for course topics
	 *  Creates receive queue for student replies 
	 *  Registers as a listener for queue messages
	 *  Starts the topic and queue connections
	 */
	
	public SchoolApp() {
		try {
			InitialContext jndi = new InitialContext();
			TopicConnectionFactory factory =
				(TopicConnectionFactory)jndi.lookup
					("java:comp/env/jms/MyTopicConnectionFactory");
			topicConnect = factory.createTopicConnection();
			publishSession =
				topicConnect.createTopicSession(false, 
					Session.AUTO_ACKNOWLEDGE);

			courseTopic = (Topic)jndi.lookup
				("java:comp/env/jms/NewCourseTopics");
			QueueConnectionFactory queueFactory =
				(QueueConnectionFactory)jndi.lookup
					("java:comp/env/jms/MyQueueConnectionFactory");
			queueConnect = queueFactory.createQueueConnection();

			queueSession =
				queueConnect.createQueueSession(false, 
					Session.AUTO_ACKNOWLEDGE);
			receiveQueue = (Queue)jndi.lookup
				("java:comp/env/jms/QueueStudents");
			publisher = publishSession.createPublisher(courseTopic);
			QueueReceiver q = queueSession.createReceiver(receiveQueue);

			q.setMessageListener(this);
			topicConnect.start();
			queueConnect.start();
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(1);
		}
	}

	/*********************************************************
	 * Publish course names, course codes, and
	 * course pricing to student subscribers
	 */
	private void publishCourses(String name, String code, int price) {
		try {
			StreamMessage message =
				publishSession.createStreamMessage();
			message.writeString(name);
			message.writeString(code);
			message.writeInt(price);

			message.setJMSReplyTo(receiveQueue);
			publisher.publish(message);

		} catch (JMSException ex) {
			ex.printStackTrace();
			System.exit(1);
		}
	}

	/**************************************************
	 * Called when queue message arrives from Student MDB
	 */
	public void onMessage(Message message) {
		try {
			TextMessage msg = (TextMessage)message;
			String text = msg.getText();
			System.out.println("Client received PTP message");
			System.out.println("\t" + text);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(1);
		}
	}

	/********************************************
	 * Close topic and queue connections
	 */
	public void exit() {
		try {
			if (topicConnect != null)
				topicConnect.close();
			if (queueConnect != null)
				queueConnect.close();
		} catch (JMSException ex) {
			ex.printStackTrace();
			System.exit(1);
		}
		System.exit(0);
	}

	/*************************************************************
	 * Main program to read Course info from the command line and
	 * publish to student subscribers 
	 * Loop until User types quit to exit program
	 */
	public static void main(String args[]) {
		String courseName, courseCode;
		int price;
		if (args.length == 3) {
			courseName = args[0];
			courseCode = args[1];
			price = Integer.parseInt(args[2]);
		} else {
			System.out.println("Usage: Course Code Price");
			return;
		}

		SchoolApp school = new SchoolApp();

		try {
			BufferedReader stdin = new
				BufferedReader(new InputStreamReader(System.in));
			school.publishCourses(courseName, courseCode, price);
			System.out.println("Published " +
				courseName + " " + courseCode +
					" Course, waiting for response");

			while (true) {
				System.out.println("type quit to exit");
				String cmd = stdin.readLine();
				if (cmd.equals("quit")) 
					school.exit();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}
