// CustomerDAOCloudscape.java

package asg.domain;

import java.util.*;
import javax.ejb.*;
import javax.naming.*;
import javax.sql.DataSource;
import java.sql.*;

// Implements CustomerDAO for Cloudscape database.
public class CustomerDAOCloudscape implements CustomerDAO {

	private Connection con = null;
	private DataSource datasource = null;

	public CustomerDAOCloudscape() 
				throws CustomerDAOSysException {
		String dbName = "java:comp/env/jdbc/CustomerDB";
		try {
			InitialContext ic = new InitialContext();
			datasource = (DataSource) ic.lookup(dbName);
		} catch (NamingException ex) {
			throw new CustomerDAOSysException("Cannot connect to database: " +
				dbName + ":\n" + ex.getMessage());
		}
	}
	
	// Utility class to generate universal unique
	// primary key
	// called by ejbCreate()
	public String dbGetKey() {
		MyKeyGen kg = MyKeyGen.getInstance();
		return kg.getNextKey();
	}

	// Obtain a JDBC Database connection
	private void getConnection() 
				throws CustomerDAOSysException {
		try {
			con = datasource.getConnection();
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"Exception during DB Connection:\n" 
				+ ex.getMessage());
		}
	}

	// Release JDBC Database connection
	private void disConnect() 
				throws CustomerDAOSysException {
		try {
			if (con != null) con.close();
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"SQLException during DB close connection:\n" + 
				ex.getMessage());
		}
	}

	private void closeStatement(PreparedStatement s) 
				throws CustomerDAOSysException {
		try {
			if (s != null) s.close();
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"SQL Exception during statement close\n" 
				+ ex.getMessage());
		}
	}

	private void closeResultSet(ResultSet rs) 
				throws CustomerDAOSysException {
		try {
			if (rs != null) rs.close();
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"SQL Exception during result set close\n" 
				+ ex.getMessage());
		}
	}

	// called by ejbCreate()
	public void dbInsertCustomer(CustomerModel newData)
			throws CreateException {
		
		String insertStatement =
			"insert into Customers values " +
			"( ? , ? , ? , ?, ? )";
		PreparedStatement prepStmt = null;
		try {
			getConnection();
			prepStmt = 
				con.prepareStatement(insertStatement);

			prepStmt.setString(1, newData.getCustomerID());
			prepStmt.setString(2, newData.getName());
			prepStmt.setString(3, newData.getPassword());
			prepStmt.setString(4, newData.getEmail());
			prepStmt.setBoolean(5, newData.getOrdersPending());

			prepStmt.executeUpdate();
		} catch (SQLException ex) {
			throw new CreateException(ex.getMessage());
		} finally {
			closeStatement(prepStmt);
			disConnect();
		}
	}

	// called by ejbFindByPrimaryKey()
	public boolean dbSelectByPrimaryKey(String primaryKey) 
				throws CustomerDAOSysException {
		
		String selectStatement =
			"select CustomerID " +
			"from Customers where CustomerID = ? ";
		PreparedStatement prepStmt = null;
		ResultSet rs = null;
		boolean result = false;
		try {
			getConnection();

			prepStmt =
				con.prepareStatement(selectStatement);
			prepStmt.setString(1, primaryKey);

			rs = prepStmt.executeQuery();
			result = rs.next();
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"dbSelectByPrimaryKey: SQL Exception caught\n" 
				+ ex.getMessage());
		
		} finally {
			closeResultSet(rs);
			closeStatement(prepStmt);
			disConnect();
		}
		return result;
	}

	// called by ejbFindByCustomerName()
	public Collection dbSelectByCustomerName(String name) 
				throws CustomerDAOSysException {
		
		System.out.println("dbSelectByCustomerName for " + name);

		String selectStatement =
			"select distinct CustomerID " +
			"from Customers where Name = ? ";
		PreparedStatement prepStmt = null;
		ResultSet rs = null;
		ArrayList a = null;
		try {
			getConnection();
			prepStmt = 
				con.prepareStatement(selectStatement);

			prepStmt.setString(1, name);
			rs = prepStmt.executeQuery();
			a = new ArrayList();

			while (rs.next()) {
				a.add(new String(rs.getString(1)));
			}

		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"dbSelectByCustomerName: SQL Exception caught\n" 
				+ ex.getMessage());
		
		} finally {
			closeResultSet(rs);
			closeStatement(prepStmt);
			disConnect();
		}
		return a;
	}
	
	// called by ejbFindAll()
	public Collection dbSelectAll() 
				throws CustomerDAOSysException {

		String selectStatement =
			"select distinct CustomerID " +
			"from Customers ";
		PreparedStatement prepStmt = null;
		ResultSet rs = null;
		ArrayList a = null;
		try {
			getConnection();
			prepStmt = 
				con.prepareStatement(selectStatement);

			rs = prepStmt.executeQuery();
			a = new ArrayList();

			while (rs.next()) {
				a.add(new String(rs.getString(1)));
			}

		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"dbSelectAll: SQL Exception caught\n" 
				+ ex.getMessage());
		
		} finally {
			closeResultSet(rs);
			closeStatement(prepStmt);
			disConnect();
		}
		return a;
	}

	// called by ejbLoad()
	public CustomerModel dbLoadCustomer(String primaryKey) 
			throws CustomerDAOSysException, NoSuchEntityException {
		String selectStatement =
			"Select Name, Password, Email, OrdersPending from Customers " +
			"Where CustomerID = ? ";
		PreparedStatement prepStmt = null;
		ResultSet rs = null;
		String name = null;
		String password = null;
		String email = null;
		boolean ordersPending = false;
		try {

			getConnection();

			prepStmt = 
				con.prepareStatement(selectStatement);

			prepStmt.setString(1, primaryKey);
			rs = prepStmt.executeQuery();

			if (rs.next()) {
				// Read the data from the resultset
				name = rs.getString(1); 				// Name
				password = rs.getString(2);			// Password
				email = rs.getString(3);				// Email
				ordersPending = rs.getBoolean(4);	// OrdersPending
			}
			else {
				throw new NoSuchEntityException("Row for customerID " + 
					primaryKey + " not found in database.");
			}
			System.out.println("dbLoadCustomer: " + name); 
			// Create CustomerModel object to return to CustomerEJB
			return new CustomerModel(primaryKey, name, password, email, ordersPending);
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"dbLoadCustomer: SQL Exception caught\n" 
				+ ex.getMessage());
		
		} finally {
			closeResultSet(rs);
			closeStatement(prepStmt);
			disConnect();
		}
	}

	// called by ejbStore()
	public void dbStoreCustomer(CustomerModel data) 
			throws CustomerDAOSysException {
		System.out.println("dbStoreCustomer: " 
					+ data.getName());
		String updateStatement =
			"update Customers set " +
			"Name = ? , " +
			"Password = ? , " +
			"Email = ? , " +
			"OrdersPending = ? " +
			"where CustomerID = ?";
		PreparedStatement prepStmt = null;
		try {
			getConnection();
			prepStmt = 
				con.prepareStatement(updateStatement);

			// Grab values from persistent fields
			// to store in database
			prepStmt.setString(1, data.getName());
			prepStmt.setString(2, data.getPassword());
			prepStmt.setString(3, data.getEmail());
			prepStmt.setBoolean(4, data.getOrdersPending());
			prepStmt.setString(5, data.getCustomerID());
			int rowCount = prepStmt.executeUpdate();

			if (rowCount == 0) {
				throw new SQLException("Storing row for CustomerID " + 
					data.getCustomerID() + " failed.");
			}
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"dbStoreCustomer: SQL Exception caught\n" 
				+ ex.getMessage());
		
		} finally {
			closeStatement(prepStmt);
			disConnect();
		}
	}
	
	// called by ejbRemove()
	public void dbRemoveCustomer(String primaryKey) 
				throws CustomerDAOSysException {
		String removeStatement =
			"delete from Customers where CustomerID = ?";
		PreparedStatement prepStmt = null;
		try {
			getConnection();
			prepStmt = 
				con.prepareStatement(removeStatement);

			prepStmt.setString(1, primaryKey);
			int result = prepStmt.executeUpdate();

			if (result == 0) {
				throw new SQLException("Remove for CustomerID " + 
					primaryKey + " failed.");
			}
		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"dbRemoveCustomer: SQL Exception caught\n" 
				+ ex.getMessage());
		
		} finally {
			closeStatement(prepStmt);
			disConnect();
		}
	}

	// Use a Select query to count 
	// the number of records in the database
	// called by ejbHomeGetTotalCustomers()
	public int dbCountTotalCustomers() 
				throws CustomerDAOSysException {

		String selectStatement =
			"select distinct CustomerID " +
			"from Customers ";
		PreparedStatement prepStmt = null;
		ResultSet rs = null;
		int count = 0;
		try {
			getConnection();

			// Request a resultset that is scrollable
			// so we can easily count rows
			prepStmt = 
				con.prepareStatement(selectStatement, 
					ResultSet.TYPE_SCROLL_INSENSITIVE, 
					ResultSet.CONCUR_UPDATABLE);
			rs = prepStmt.executeQuery();
			// go to the last row
			// and get its row number
			rs.last();
			count = rs.getRow();

		} catch (SQLException ex) {
			throw new CustomerDAOSysException(
				"dbCountTotalCustomers: SQL Exception caught\n" 
				+ ex.getMessage());
		
		} finally {
			closeResultSet(rs);
			closeStatement(prepStmt);
			disConnect();
		}
		return count;
	}
	
} // CustomerDAOCloudscape
