// MusicApp.java

// Swing application that reads the Music Collection Database
// using the Music EJB stateless session bean
// Puts recording titles into a Swing JList component
// so that user can select the title
// Upon selection (ListSelectionEvent), 
// displays additional information about the recording
// When user clicks "View Tracks" JButton (ActionEvent),
// displays track list for the selected recording title.

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import asg.MusicEJB.*;

public class MusicApp extends JFrame implements 
	ActionListener, ListSelectionListener {

	private MusicHome musicHome;		// Music EBJ home interface
	private Music mymusic;			// Music EBJ remote interface

	// Swing components
	private JList musicTitle;		// Display music titles, selectable
	private JTextArea data;			// Display recording data or track titles
	private JPanel statusbar;		// Panel to hold status text field
	private JPanel display;			// Panel to hold music & track titles
	private JLabel helpline;			// Tell people how to use the program
	private JPanel helpPanel;		// Panel to hold helpline
	private JTextField status;		// Give user status, error messages
	private JButton goButton;		// Button to view tracks

	// Keep track of various lists
	private ArrayList albums;		// Collection of RecordingVO objects
	private ArrayList tracks;		// Collection of TrackVO objects
	private Vector titles;  			// Collection of recording titles

	// Given a RecordingVO, get the tracklist
	private void getTracks(RecordingVO rec) {
		try {
			// access MusicEJB
			tracks = mymusic.getTrackList(rec);
			TrackVO t;
			Iterator i = tracks.iterator();
			while (i.hasNext()) {
				t = (TrackVO)i.next();
				// put track info into data JTextArea
				data.append("\n" +  t.getTrackNumber() + "\t" +
					t.getTrackLength() + "\t" + t.getTitle());
			}
		// Check separately for NoTrackListException
		} catch (NoTrackListException ex) {
			status.setText("NoTrackListException Exception: " + 
			ex.getMessage());
			ex.printStackTrace();
		} catch (Exception ex) {
			status.setText("Unexpected Exception: " + 
			ex.getMessage());
			ex.printStackTrace();
		}
	}

	// Get recording information
	private void getRecordings() {
		try {
			// access MusicEJB
			albums = mymusic.getMusicList();
		} catch (Exception ex) {
			status.setText("Unexpected Exception: " + 
			ex.getMessage());
			ex.printStackTrace();
		}
		RecordingVO r;
		Iterator i = albums.iterator();
		while (i.hasNext()) {
			r = (RecordingVO)i.next();
			// put Recording titles into titles Vector,
			// which is displayed in the JList Swing component
			titles.add(r.getTitle());
		}
	}

	public MusicApp() {

		// setup Swing components
		getContentPane().setLayout(new BorderLayout());
		getContentPane().setBackground(Color.lightGray);
		setSize(800, 200);

		helpPanel = new JPanel();
		helpPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));

		// Put the help panel at the top
		getContentPane().add("North", helpPanel);
		helpline = new JLabel(
			"Select recording & click button to View Tracks");
		helpPanel.add(helpline);

		display = new JPanel();
		display.setLayout(new GridLayout(1,2,15,15));

		// Set up the musicTitle JList
		// Initialize it with titles vector
		// Make 7 rows visitble and add a scrollbar
		// Add the ListSelectionListener 
		// to allow user to select titles
		titles = new Vector();
		musicTitle = new JList(titles);
		musicTitle.setBackground(Color.white);
		musicTitle.setVisibleRowCount(7);
		musicTitle.addListSelectionListener(this);
		display.add( new JScrollPane( musicTitle ));

		// Set up the data JTextArea
		// Turn off editing and add a scrollbar
		data = new JTextArea(12,40);
		data.setBackground(Color.white);
		data.setEditable(false);
		display.add( new JScrollPane( data ));

		// Put the display JPanel in the Center
		getContentPane().add("Center", display);

		// Set up the statusbar JPanel
		// Put the statusbar JPanel at the bottom of the application
		// Give it a flow layout
		// Add the goButton JButton and the status JTextField
		// Add the Action Listener to goButton to detect user selection
		statusbar = new JPanel();
		statusbar.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
		goButton = new JButton("View Tracks");
		goButton.addActionListener(this);
		statusbar.add(goButton);

		status = new JTextField(60);
		status.setEditable(false);
		statusbar.add(status);
		getContentPane().add("South", statusbar);
		setTitle("Music Collection - Using EJB with JDBC, Vers. 3");

		try {
			// Create Music EJB
			Context initial = new InitialContext();
			Object objref = initial.lookup("java:comp/env/ejb/MyMusic");
			musicHome = (MusicHome)PortableRemoteObject.narrow(objref, 
			MusicHome.class);
			mymusic = musicHome.create();
		} catch (Exception ex) {
			System.out.println("Unexpected Exception: " + 
			ex.getMessage());
			ex.printStackTrace();
		}

		// Access the EJB to read the Music Collection database
		// and put the title names in the JList swing component
		getRecordings();
		status.setText("There are " + albums.size() + " recordings");
		pack();
		show();
		
		// Set up the WindowListener object
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) 
				{ dispose(); }
			public void windowClosed(WindowEvent e) 
				{ System.exit(0); }
		});
	}

	public MusicApp(String title) {
		this();
		setTitle(title);
	}

	public void actionPerformed(ActionEvent e) {

		// Event handler for ActionEvent 
		// (user clicks "View Tracks" button)

		if (e.getSource() instanceof JButton) {

		// make sure the user has selected a music title
			if (musicTitle.getSelectedValue() == null) {
				status.setText("You must select a recording title first.");
			}
			else {
				// Tell the user what title we're getting track info for
				status.setText("Displaying track list for " + 
					musicTitle.getSelectedValue());
				// Using setText() wipes out any text previously
				// written to JTextArea
				data.setText("Track information for " + 
					musicTitle.getSelectedValue());

				// getTracks calls the MusicEJB business method and
				// puts track information into JTextArea component
				getTracks(
					(RecordingVO)albums.get(musicTitle.getSelectedIndex()));

				// In case there are more tracks than visible lines,
				// position caret position to top of component
				data.setCaretPosition(0);
			}
		}
	}

	public void valueChanged(ListSelectionEvent e) {

		// Event handler for ListSelectionEvent 
		// (user changes selection in musicTitle JList component)

		if (!e.getValueIsAdjusting()) {

			// Tell user which title was selected
			// Method getSelectedValue() returns a String
			status.setText("You selected " + 
				musicTitle.getSelectedValue());

			// Access the RecordingVO object at the index
			// of the selected music title.
			// Method getSelectedIndex() returns the index of
			// the selected list item, which corresponds to the same
			// index value in the albums ArrayList collection.
			RecordingVO r = 
				(RecordingVO)albums.get(musicTitle.getSelectedIndex());

			// Display information from the RecordingVO object
			// in the JTextArea.
			// Use method setText() to clear previously written text,
			// then use method append().
			// Set caret position to top in case text causes JTextArea
			// to scroll.
			data.setText("RecordingID = " + r.getRecordID() + ",\n");
			data.append("Title = " + r.getTitle() + ",\n");
			data.append("Artist is " + r.getArtistName() + ",\n");
			data.append("Music Category is " + r.getMusicCategory() + ",\n");
			data.append("Label is " + r.getLabel() + ",\n");
			data.append("Number of Tracks = " + r.getNumberOfTracks());
			data.setCaretPosition(0);
		}
	}

	static public void main(String args[]) {
		MusicApp acs = new MusicApp();
	}
}
