// MusicBean.java

package asg.MusicEJB;

import java.sql.*;
import javax.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;
import java.rmi.RemoteException; 

public class MusicBean implements SessionBean {

	// Business methods

	public ArrayList getMusicList() 
	{
		try {
			// Encapsulate database calls in separate method
			return (dbLoadMusicList());
		} catch (Exception ex) {
			throw new EJBException("getMusicList: " + ex.getMessage());
		}
	}

	public ArrayList getTrackList(RecordingVO rec) 
		throws NoTrackListException {

		ArrayList trackList;
		try {
			// Encapsulate database calls in separate method
			trackList = dbLoadTrackList(rec);
		} catch (Exception ex) {
			throw new EJBException("getTrackList: " + ex.getMessage());
		}
		if (trackList.size() == 0) {
			throw new NoTrackListException(
				"No Track List found for RecordingID " + rec.getRecordID());
		}
		return trackList;
	}

	private Connection con;			// JDBC Database Connection
	private DataSource ds;			// DataSource
	
	// EJB methods

	// Do the DataSource lookup just once

	public void ejbCreate() {
		String dbName = "java:comp/env/jdbc/MusicDB";
		try {
			InitialContext ic = new InitialContext();
			ds = (DataSource) ic.lookup(dbName);
		} catch (Exception ex) {
			throw new EJBException("Cannot find DataSource: " +
				ex.getMessage());
		}
	}

	public MusicBean() {}
	public void ejbRemove() {}
	public void ejbActivate() {}
	public void ejbPassivate() {}
	public void setSessionContext(SessionContext sc) {}

	// Database methods
	
	// Obtain a JDBC Database connection
	private void getConnection() 
	{
		try {
			con = ds.getConnection();
		} catch (Exception ex) {
			throw new EJBException("Cannot connect to database: " +
				ex.getMessage());
		}
	}

	// Release JDBC Database connection
	private void disConnect() 
	{
		try {
			if (con != null) con.close();
		} catch (SQLException ex) {
			throw new EJBException("disConnect: " + ex.getMessage());
		}
	}

	private ArrayList dbLoadMusicList() throws SQLException
	{
		ArrayList mList = new ArrayList();

		// Objects needed for the JDBC API SQL calls:
		PreparedStatement musicStmt = null;
		ResultSet rs = null;

		// The following query searches 3 tables in
		// the Music Collection database to create a composite
		// RecordingVO object.
		// We select all fields in the Recordings table,
		// the RecordingArtistName field in the Recording Artists table,
		// and the MusicCategory field from the Music Categories table.
		// The Where clause matches the correct records from the
		// Recording Artists and Music Categories tables.
		
		String selectQuery = "Select Recordings.*, " +
			"\"Recording Artists\".RecordingArtistName,  " +
			"\"Music Categories\".MusicCategory " +
			"From Recordings, \"Recording Artists\", \"Music Categories\" " +
			"Where Recordings.RecordingArtistID = " +
			"\"Recording Artists\".RecordingArtistID and  " +
			"Recordings.MusicCategoryID = " +
			"\"Music Categories\".MusicCategoryID ";

		try {
			// Obtain a database connection
			getConnection();
		
			musicStmt = con.prepareStatement(selectQuery);
			
			// Execute the query; results are in ResultSet rs
			rs = musicStmt.executeQuery();

			// Loop through ResultSet and create RecordingVO object
			// for each row
			while (rs.next())
			{
				// Create RecordingVO object and add to mList ArrayList
				// Use getInt() and getString() to pull data from
				// ResultSet rs.
				// The parameter numbers match the order of the
				// field names in the query statement.
				mList.add(new RecordingVO(
					rs.getInt(1),			// RecordingID
					rs.getString(2),		// RecordingTitle
					rs.getString(9),		// RecordingArtistName
					rs.getString(10),		// MusicCategory
					rs.getString(5),		// RecordingLabel
					rs.getInt(7)));			// NumberofTracks
			}
		} catch (SQLException ex) {
			throw new EJBException(
				"SQLException reading recording data:\n" + 
				ex.getMessage());
		} finally {
			// Close the ResultSet and the PreparedStatement objects
			if (rs != null) rs.close();
			if (musicStmt != null) musicStmt.close();
			
			// Release the database connection
			disConnect();
		}
		return mList;
	}

	private ArrayList dbLoadTrackList(RecordingVO rec) throws SQLException
	{
		ArrayList tList = new ArrayList();
		
		// Objects needed for the JDBC API SQL calls:
		PreparedStatement trackStmt = null;
		ResultSet rs = null;

		// Select all records from Tracks table
		// where the RecordingID is the same as the In parameter
		// Order the records by TrackNumber field
		
		String selectQuery = "Select * From Tracks " +
			"Where RecordingID = ? Order By TrackNumber";
		
		try {
			// Obtain a database connection
			getConnection();

			// Get a PreparedStatement with query
			trackStmt = con.prepareStatement(selectQuery);

			// Set In parameter to RecordingID in RecordingVO argument rec
			trackStmt.setInt(1, rec.getRecordID());

			// Execute query; return in ResultSet rs
			rs = trackStmt.executeQuery();
			while (rs.next())
			{
				// Loop through ResultSet and create TrackVO object
				// Add to tList ArrayList
				tList.add(new TrackVO(
				rs.getInt(2),		// trackNumber
				rs.getString(3),	// title
				rs.getString(4)));	// trackLength
			}
		} catch (SQLException ex) {
			throw new EJBException(
				"SQLException reading track data:\n" + 
				ex.getMessage());
		} finally {
			if (rs != null) rs.close();
			if (trackStmt != null) trackStmt.close();
			
			// Release the database connection
			disConnect();
		}
		return tList;
	}
    
} // MusicBean
